<?php

namespace InSegment\ApiCore\Traits;

use Illuminate\Support\Facades\App;

/**
 * To be used with \Illuminate\Database\Eloquent\Model
 */
trait Instantiatible
{
    /**
     * Information about the function and the model instantiating this model
     * Only available on 'instantiated' event fire
     * 
     * @var [
     *     'model' => \Illuminate\Database\Eloquent\Model,
     *     'funciton' => __FUNCTION__,
     *     ...
     * ]|null
     */
    public $_instantiator;
    
    /**
     * Boot method
     * 
     * @return null
     */
    public static function bootInstantiatible()
    {
        /**
         * Tap into the eloquent booted event so we can add the extra observable event.
         * This will allow the use of observers for archiving event instead of static bindings.
         */
        App::make('events')->listen('eloquent.booted: '.static::class, function($model){
            $model->addObservableEvents('instantiated');
        });
    }
    
    /**
     * Register an instantiated model event with the dispatcher.
     *
     * @param  \Closure|string  $callback
     * @return void
     */
    public static function instantiated($callback)
    {
        static::registerModelEvent('instantiated', $callback);
    }
    
    /**
     * Fire instantination event with data
     * 
     * @param array $instantiator
     * @return bool
     */
    public function fireInstantination($instantiator)
    {
        $this->_instantiator = $instantiator;
        $ret = $this->fireModelEvent('instantiated');
        $this->_instantiator = null;
        return $ret;
    }
    
    /**
     * Fire instantiated event on newInstance
     * 
     * @param array $attributes
     * @param bool $exists
     * @return static
     */
    public function newInstance($attributes = [], $exists = false)
    {
        $newModel = parent::newInstance($attributes, $exists);
        $newModel->fireInstantination([
            'model' => $this,
            'function' => __FUNCTION__,
            'attributes' => &$attributes,
            'exists' => &$exists
        ]);
        return $newModel;
    }
    
    /**
     * Fire instantiated event on newFromBuilder
     * 
     * @param array $attributes
     * @param \Illuminate\Database\Connection|null $connection
     * @return static
     */
    public function newFromBuilder($attributes = [], $connection = null) {
        $newModel = parent::newFromBuilder($attributes, $connection);
        $newModel->fireInstantination([
            'model' => $this,
            'funciton' => __FUNCTION__,
            'attributes' => &$attributes,
            'connection' => $connection
        ]);
        return $newModel;
    }
    
}
